/*********************************
 *
 * Affine Mode, Demo 1
 *
 *  Based on parasytes rotate demo
 *
 *  By david Tucker
 *  Rev: 0.9, 8/25/05 - initial release
 *
 *  Demonstrates several of the basic affine
 *   mode transforms and introduces the tools
 *   for affine mode manipulations.
 *
 **********************************/

//vbgcc includes
#include "types.h"
#include "video.h"
#include "joypad.h"
#include "world.h"
#include "mem.h"

//dt includes
#include "dt_math.h"
#include "dt_affine.h"

//images
#include "focus_map.h"
#include "mario_map.h"

//we iterate two varialbes to controll scale and rotation
// this sets the limits on the varialbes
#define SCLSTEP  1.02f
#define maxscale 10.0f
#define minscale 0.05f
#define maxalpha 511

//define the display
#define screenW 384
#define screenH 224
#define screenW_Center (screenW/2)
#define screenH_Center (screenH/2)

//define our image to transform
#define imageW 128
#define imageH 104
#define imageW_Center (imageW/2)
#define imageH_Center (imageH/2)

//memory base of the param table, don't overwrite with char data!
u16* const param = (u16*)0x0003C000;

//Display the focus screen and wait for keyboard input
void focus_screen() {
	int i;

	//initiate left focus screen
	WA[31].head = WRLD_LON;
	WA[31].gx = 0;
	WA[31].gp = 0;
	WA[31].gy = 0;
	WA[31].mx = 0;
	WA[31].mp = 0;
	WA[31].my = 0;
	WA[31].w  = screenW;
	WA[31].h  = screenH;

	//initiate right focus screen
	WA[30].head = WRLD_RON;
	WA[30].gx = 0;
	WA[30].gp = 0;
	WA[30].gy = 0;
	WA[30].mx = 0;
	WA[30].mp = 0;
	WA[30].my = screenH;
	WA[30].w  = screenW;
	WA[30].h  = screenH;

	//last world
	WA[29].head = WRLD_END;

	//move image to char table and bgmap
	copymem((BYTE*)CharSeg0, focus_char, focus_char_len);
	copymem((BYTE*)BGMMBase, focus_map, focus_map_len);

	//turn on display
	vbDisplayOn();
	vbDisplayHide();

	vbFXFadeIn(1);

	//do we need to put a delay in?
	while(! vbPadKeyDown()) ;
	while(! vbPadKeyUp()  ) ;

	vbFXFadeOut(1);
}

//setup for affine mode, could be cleaner
void initialize() {
	//initiate world as affine mode
	WA[31].head  = WRLD_ON | WRLD_AFFINE | WRLD_OVR;
	WA[30].gx    = 0;
	WA[30].gp    = 0;
	WA[30].gy    = 0;
	//it would be more efficient to set w/h based on the scale
	//factor, rather thain just forcing it to the screen dimentions
	//but that requires math =0)
	WA[31].w     = screenW;
	WA[31].h     = screenH;
	WA[31].param = (u32)param;
	//blank overplane character, kills BG tiling
	WA[31].ovr   = 0; 
	WA[30].head  = WRLD_END;

	//move image to char table
	copymem((BYTE*)CharSeg0, mario_char, mario_char_len);
	//clear BGMap and Fill it in, arg! why can't our
	// graphics tool put a blank char at index 0
	setmem((BYTE*)BGMap(0), 145, 0x2000);
	copymem((BYTE*)BGMap(0), mario_map, mario_map_len);

	//clear param table, broken
	//affine_clear_all(31);
	vbDisplayShow();
}

int main () {
	PDx_ST pdx;
	//warning*** dont get caught with the bad naming!
	float sclstep = SCLSTEP;
	float zoom=minscale;
	int alpha = 0;
	int index;

	focus_screen();
	initialize();

	while (1) {
		//Zoom by a mulcaplicitive constant so closer objects move
		//faster....
		zoom *= sclstep;
		if((zoom>maxscale)||(zoom<minscale)) {
			sclstep = 1.0f/sclstep;
		}

		//increment (rotation)
		if(alpha++ > maxalpha) alpha = 0;

		//pick an affine effect
		switch(index) {
		case 0:
			affine_rotscl(31,alpha,zoom,
				 imageW_Center, imageH_Center,
				 screenW_Center, screenH_Center);
			break;
		case 1:
			affine_rotate(31,alpha,
				 imageW_Center, imageH_Center,
				 screenW_Center, screenH_Center);
			break;
		case 2:
			affine_scale(31,zoom,
				 imageW_Center, imageH_Center,
				 screenW_Center, screenH_Center);
			break;
		case 3:
			//test bed
			pdx.pa = 1;
			pdx.pb = 0.5f;
			pdx.pc = 0;
			pdx.pd = 1;

			pdx.dx = imageW_Center-(pdx.pa*screenW_Center+pdx.pb*screenH_Center);
			pdx.dy = imageH_Center-(pdx.pc*screenW_Center+pdx.pd*screenH_Center);
			
			pdx.paralax = 0;

			affine_set_all(31,&pdx);
			break;
		}

		//delay 1/50th of a second
		vbWaitFrame(1);

		//switch mode?
		if(vbPadKeyDown()) {
			//debounce
			while(!vbPadKeyUp()) ;
			index++;
			index &=3;
		}
	}
}
