#ifndef DT_AFFINE_H
#define DT_AFFINE_H 1

#include "types.h"
#include "video.h"

#include "dt_math.h"
#include "dt_affine.h"


typedef struct AFFINE_ST {
    fix13_3	pb_y;		// *y+Dx /= 8.0
    s16		paralax;
    fix13_3	pd_y;		// *y+Dy /= 8.0
    fix7_9	pa;			// /=512.0
    fix7_9	pc;			// /=512.0
    u16 spacer[3];		//unknown
} AFFINE_ST ;

typedef struct PDx_ST {
	float pa;
	float pb;
	float pc;
	float pd;
	float dx;
	float dy;
	s16   paralax;
} PDx_ST ;

void affine_set_all(u8 world, PDx_ST * pdx) {
	s16 i, max;
	AFFINE_ST *affine;

	affine = (AFFINE_ST*)((WA[world].param<<1)+0x00020000);
	max = WA[world].h;

	for (i = 0; i < max; i++) {
		affine[i].pb_y    = FTOFIX13_3(i*pdx->pb+pdx->dx);
		affine[i].paralax = pdx->paralax;
		affine[i].pd_y    = FTOFIX13_3(i*pdx->pd+pdx->dy);
		affine[i].pa      = FTOFIX7_9(pdx->pa);
		affine[i].pc      = FTOFIX7_9(pdx->pc);
	}
}

//***FixMe, modify theas to update GX,GY to fit the image
// This means we need an image structure to contain the
// dimentions of the image, also add a copy_map fn to 
// move a map into the map table and relocate the char pointer
// as needed.
void affine_rotscl(u8 world,s16 alpha, float zoom,
					 s16 bg_x, s16 bg_y, s16 fg_x, s16 fg_y) {
	PDx_ST pdx;
	pdx.pb  = 0.0f;
	
	pdx.pa  = COSF(alpha)*(1.0f/zoom);
	pdx.pb -= SINF(alpha)*(1.0f/zoom);
	pdx.pc  = SINF(alpha)*(1.0f/zoom);
	pdx.pd  = pdx.pa;

	pdx.dx = bg_x-(pdx.pa*fg_x + pdx.pb*fg_y);
	pdx.dy = bg_y-(pdx.pc*fg_x + pdx.pd*fg_y);
	
	pdx.paralax = 0;

	affine_set_all(world,&pdx);
}

void affine_rotate(u8 world,s16 alpha,
					 s16 bg_x, s16 bg_y, s16 fg_x, s16 fg_y) {
	PDx_ST pdx;
	pdx.pb  = 0.0f;
	
	pdx.pa  = COSF(alpha);
	pdx.pb -= SINF(alpha);
	pdx.pc  = SINF(alpha);
	pdx.pd  = pdx.pa;

	pdx.dx = bg_x-(pdx.pa*fg_x + pdx.pb*fg_y);
	pdx.dy = bg_y-(pdx.pc*fg_x + pdx.pd*fg_y);
	
	pdx.paralax = 0;

	affine_set_all(world,&pdx);
}

void affine_scale(u8 world, float zoom,
					 s16 bg_x, s16 bg_y, s16 fg_x, s16 fg_y) {
	PDx_ST pdx;
	
	pdx.pa  = 1.0f/zoom;
	pdx.pb  = 0.0f;
	pdx.pc  = 0.0f;
	pdx.pd  = 1.0f/zoom;

	pdx.dx = bg_x-(pdx.pa*fg_x + 0);
	pdx.dy = bg_y-(0           + pdx.pd*fg_y);

	pdx.paralax = 0;

	affine_set_all(world,&pdx);
}


//#define affine_clear_all(w) affine_set_all(w, 1.0f,0.0f,0.0f,1.0f,0.0f,0.0f,0)


#endif // DT_AFFINE_H